import os
import time

import pytest
from charli3_dendrite import MinswapCPPState
from dotenv import load_dotenv
from pycardano import Address

from kupo import KupoClient

load_dotenv()


@pytest.fixture
def kupo_client():

    url = (
        "http://localhost"
        if os.environ.get("KUPO_BASE_URL") is None
        else os.environ.get("KUPO_BASE_URL")
    )

    port = (
        1442
        if os.environ.get("KUPO_PORT") is None
        else int(os.environ.get("KUPO_PORT"))
    )

    return KupoClient(base_url=url, port=port)


@pytest.fixture(params=[True, False])
def unspent(request):
    return request.param


@pytest.fixture(params=[True, False])
def spent(request):
    return request.param


@pytest.fixture(params=[None, time.time() - 3600 * 24 * 2, time.time() - 3600 * 24])
def spent_after(request):
    return request.param


@pytest.fixture(params=[None, time.time() - 3600 * 24 * 2, time.time() - 3600 * 24])
def created_after(request):
    return request.param


@pytest.fixture(params=[None, time.time()])
def spent_before(request):
    return request.param


@pytest.fixture(params=[None, time.time()])
def created_before(request):
    return request.param


patterns: list[str] = []

address = Address.decode(MinswapCPPState.pool_selector().addresses[0])

patterns.append(address.payment_part.payload.hex() + "/*")


@pytest.fixture(params=patterns)
def pattern(request):
    return request.param



# noqa

from kupo.models import Point

ERA_BOUNDARIES = {
    "last_byron_block": Point(
        slot_no=4492799,
        header_hash="f8084c61b6a238acec985b59310b6ecec49c0ab8352249afd7268da5cff2a457",
    ),
    "last_shelley_block": Point(
        slot_no=16588737,
        header_hash="4e9bbbb67e3ae262133d94c3da5bffce7b1127fc436e7433b87668dba34c354a",
    ),
    "last_allegra_block": Point(
        slot_no=23068793,
        header_hash="69c44ac1dda2ec74646e4223bc804d9126f719b1c245dadc2ad65e8de1b276d7",
    ),
    "last_mary_block": Point(
        slot_no=39916796,
        header_hash="e72579ff89dc9ed325b723a33624b596c08141c7bd573ecfff56a1f7229e4d09",
    ),
    "last_alonzo_block": Point(
        slot_no=72316796,
        header_hash="c58a24ba8203e7629422a24d9dc68ce2ed495420bf40d9dab124373655161a20",
    ),
}



import pytest


def patterns(benchmark, kupo_client):
    benchmark(kupo_client.get_patterns)


@pytest.mark.asyncio
async def test_get_patterns(kupo_client):

    await kupo_client.get_patterns_async()



from kupo import KupoClient
from kupo.client import Limit
from kupo.utils import ERA_BOUNDARIES

from charli3_dendrite.dexs.core.base import AbstractPairState
from pycardano import Address

# Get all DEX classes
subclass_walk = [AbstractPairState]
D: list[AbstractPairState] = []

while len(subclass_walk) > 0:
    c = subclass_walk.pop()

    subclasses = c.__subclasses__()

    # If no subclasses, this is a a DEX class. Ignore MuesliCLP for now
    try:
        if isinstance(c.dex(), str) and c.__name__ not in ["MuesliSwapCLPState"]:
            c.pool_selector()
            D.append(c)
            subclass_walk.extend(subclasses)
        else:
            subclass_walk.extend(subclasses)
    except NotImplementedError:
        subclass_walk.extend(subclasses)

D = list(sorted(set(D), key=lambda d: d.__name__))

# Get currently tracked patterns
client = KupoClient(
    base_url="http://kupo-z4wswcco0wco008s800s4g0g.167.235.164.240.sslip.io", port=80
)


current_patterns = [p.root for p in client.get_patterns()]
# current_patterns = []

print(current_patterns)

# Generate a list of all pool scripts not currently tracked
pool_scripts = []
for d in D:
    if d.dex() == "Axo":
        continue
    addresses = d.pool_selector().addresses
    for address in addresses:
        script_hash = Address.decode(address).payment_part.payload.hex()
        pattern = f"{script_hash}/*"

        if pattern in current_patterns:
            print(f"Pattern for {d.dex()} is already being tracked: {pattern}")
            continue
        pool_scripts.append(f"{script_hash}/*")

pool_scripts = list(set(pool_scripts))

print(f"New patterns to track: {len(pool_scripts)}")

print(pool_scripts)

# Start tracking untracked patterns
for pattern in pool_scripts:
    patterns = client.add_pattern(
        pattern=pattern,
        rollback_to=ERA_BOUNDARIES["last_mary_block"],
        limit=Limit.UNSAFE,
    )
# print(current_patterns)
